#include <mbed.h>
#include "Mini_DK.h"

extern "C" {
#include <cr_dsplib.h>
}

__attribute((section("AHBSRAM0"),aligned)) static short fft_input[2048];
__attribute((section("AHBSRAM0"),aligned)) static short fft_out[2048];
__attribute((section("AHBSRAM0"),aligned)) static float window[1024];

DigitalOut led(DK_LED1);
AnalogIn   ain(P0_26);

SPI_TFT TFT(P0_18, P0_17, P0_15, P0_16, NC, "TFT");

inline float hann(int n, int N) {
	// -32    Highest Side-Lobe (dB)
	// -18    Side-Lobe Roll-Off (dB/octave)
	//   1.44 3.0-dB Bandwidth	(1.0/T)
	//   3.18 Worst Case Process Loss (dB)

	return 0.5 * (1 - cos(2*M_PI*n / (N-1)));
}

inline float blackman_harris(int n, int N) {
	// -92    Highest Side-Lobe (dB)
	//  -6    Side-Lobe Roll-Off (dB/octave)
	//   1.90 3.0-dB Bandwidth	(1.0/T)
	//   3.85 Worst Case Process Loss (dB)

	return 0.35875 
		-0.48829 * (cos(2 * M_PI * n / (N-1)))
		+0.14128 * (cos(4 * M_PI * n / (N-1)))
		-0.01168 * (cos(6 * M_PI * n / (N-1)));
}

int main() {

    TFT.background(Black);    // set background to black
    TFT.foreground(White);    // set chars to white
	
	TFT.cls();

    TFT.set_font(const_cast<unsigned char*>(Arial28x28));
    TFT.set_orientation(1);

	int i = 0, c = 0;

	TFT.line(0, 120, 319, 120, White);

//#define WINDOW hann
#define WINDOW blackman_harris

	for (i = 0; i < 1024; i++) {
		window[i] = WINDOW(i, 1024);
	}

	int idx = 0;

    while (1) {
		// led = !led;
		
		for (i = 0; i < 256; i++) {
			fft_input[2 * idx] = window[idx] * ((ain.read_u16() - 0x7FFF) / 50);
			fft_input[(2 * idx) + 1] = 0;
			wait_us(1000);

			if (idx++ == 1024) {
				idx = 0;
			}
		}

		vF_dspl_fftR4b16N1024(fft_out, fft_input);

		int j = 2;

#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)>(b))?(a):(b))
#define F 4

		for (i = 0; i < 320; i++) {
			int m = (int)sqrt((pow(fft_out[2 * j], 2) + pow(fft_out[(2 * j) + 1], 2))); // m = sqrt(a^2 + b^2)
			
			TFT.line(i, 0, i, MAX(0, 119 - m), Black);
			TFT.line(i, MAX(0, 119 - m), i, 119, Red);

			TFT.pixel(i, 121+c, RGB(MIN(255,m*F), MIN(255, m*F), MIN(255, m*F)));

			j++;
		}

		c++;
		c = c % 118;
	}
}
